# Copyright 2026 EcoFuture Technology Services LLC and contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Generated by Django 5.0.11 on 2025-07-28 14:02

from django.db import migrations, models
from django.utils.translation import gettext_lazy as _



class Migration(migrations.Migration):

    dependencies = [
        ('bg', '0002_alter_task_file'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='phase',
            field=models.CharField(max_length=1024, verbose_name='Phase'),
        ),
    ]
